@@ -1,6 +1,6 @@
 
 /*
-   Copyright (c) 2003-2004 Clarence Dang <dang@kde.org>
+   Copyright (c) 2003,2004,2005 Clarence Dang <dang@kde.org>
    All rights reserved.
 
    Redistribution and use in source and binary forms, with or without
@@ -152,8 +152,7 @@ bool kpGrip::isDrawing () const
 // public
 QString kpGrip::haventBegunDrawUserMessage () const
 {
-    // TODO: "Left drag..." to be consistent
-    return i18n ("Drag the handle to resize the image.");
+    return i18n ("Left drag the handle to resize the image.");
 }
 
 
@@ -394,6 +393,7 @@ kpViewScrollableContainer::kpViewScrollableContainer (kpMainWindow *parent,
                                                       const char *name)
     : QScrollView ((QWidget *) parent, name, Qt::WStaticContents | Qt::WNoAutoErase),
       m_mainWindow (parent),
+      m_contentsXSoon (-1), m_contentsYSoon (-1),
       m_view (0),
       m_bottomGrip (new kpGrip (kpGrip::Bottom, viewport (), "Bottom Grip")),
       m_rightGrip (new kpGrip (kpGrip::Right, viewport (), "Right Grip")),
@@ -435,6 +435,25 @@ kpViewScrollableContainer::~kpViewScrollableContainer ()
 }
 
 
+// public
+int kpViewScrollableContainer::contentsXSoon ()
+{
+    if (m_contentsXSoon < 0)
+        return contentsX ();
+    else
+        return m_contentsXSoon;
+}
+
+// public
+int kpViewScrollableContainer::contentsYSoon ()
+{
+    if (m_contentsYSoon < 0)
+        return contentsY ();
+    else
+        return m_contentsYSoon;
+}
+
+
 // protected
 void kpViewScrollableContainer::connectGripSignals (kpGrip *grip)
 {
@@ -495,8 +514,8 @@ QSize kpViewScrollableContainer::newDocSize (int viewDX, int viewDY) const
     if (!docResizingGrip ())
         return QSize ();
 
-    const int docX = m_view->zoomViewToDocX (m_view->width () + viewDX);
-    const int docY = m_view->zoomViewToDocY (m_view->height () + viewDY);
+    const int docX = (int) m_view->transformViewToDocX (m_view->width () + viewDX);
+    const int docY = (int) m_view->transformViewToDocY (m_view->height () + viewDY);
 
     return QSize (QMAX (1, docX), QMAX (1, docY));
 }
@@ -719,8 +738,8 @@ void kpViewScrollableContainer::updateResizeLines (int viewX, int viewY,
 
     if (viewX >= 0 && viewY >= 0)
     {
-        m_resizeRoundedLastViewX = m_view->zoomDocToViewX (m_view->zoomViewToDocX (viewX));
-        m_resizeRoundedLastViewY = m_view->zoomDocToViewY (m_view->zoomViewToDocY (viewY));
+        m_resizeRoundedLastViewX = (int) m_view->transformDocToViewX ((int) m_view->transformViewToDocX (viewX));
+        m_resizeRoundedLastViewY = (int) m_view->transformDocToViewY ((int) m_view->transformViewToDocY (viewY));
 
         m_resizeRoundedLastViewDX = viewDX;
         m_resizeRoundedLastViewDY = viewDY;
@@ -796,8 +815,8 @@ void kpViewScrollableContainer::slotGripContinuedDraw (int inViewDX, int inViewD
 
     m_haveMovedFromOriginalDocSize = true;
 
-    updateResizeLines (QMAX (1, QMAX (m_view->width () + viewDX, m_view->zoomDocToViewX (1))),
-                       QMAX (1, QMAX (m_view->height () + viewDY, m_view->zoomDocToViewY (1))),
+    updateResizeLines (QMAX (1, QMAX (m_view->width () + viewDX, (int) m_view->transformDocToViewX (1))),
+                       QMAX (1, QMAX (m_view->height () + viewDY, (int) m_view->transformDocToViewY (1))),
                        viewDX, viewDY);
 
     emit continuedDocResize (newDocSize ());
@@ -876,7 +895,6 @@ void kpViewScrollableContainer::recalculateStatusMessage ()
     //       any grip.  This also handles the case of cancelling over any
     //       grip.
     //
-    // TODO: "Left drag..." to be consistent
     if (kpWidgetMapper::toGlobal (this,
                                   QRect (0, 0, visibleWidth (), visibleHeight ()))
             .contains (QCursor::pos ()))
@@ -885,19 +903,19 @@ void kpViewScrollableContainer::recalculateStatusMessage ()
             m_bottomRightGrip->hotRect (true/*to global*/)
                 .contains (QCursor::pos ()))
         {
-            m_bottomRightGrip->setUserMessage (i18n ("Drag the handle to resize the image."));
+            m_bottomRightGrip->setUserMessage (i18n ("Left drag the handle to resize the image."));
         }
         else if (m_bottomGrip->isShown () &&
                 m_bottomGrip->hotRect (true/*to global*/)
                     .contains (QCursor::pos ()))
         {
-            m_bottomGrip->setUserMessage (i18n ("Drag the handle to resize the image."));
+            m_bottomGrip->setUserMessage (i18n ("Left drag the handle to resize the image."));
         }
         else if (m_rightGrip->isShown () &&
                 m_rightGrip->hotRect (true/*to global*/)
                     .contains (QCursor::pos ()))
         {
-            m_rightGrip->setUserMessage (i18n ("Drag the handle to resize the image."));
+            m_rightGrip->setUserMessage (i18n ("Left drag the handle to resize the image."));
         }
         else
         {
@@ -921,15 +939,20 @@ void kpViewScrollableContainer::slotContentsMoving (int x, int y)
                << " contentsY=" << contentsY () << endl;
 #endif
 
+    m_contentsXSoon = x, m_contentsYSoon = y;
+    emit contentsMovingSoon (m_contentsXSoon, m_contentsYSoon);
+
     // Reduce flicker - don't let QScrollView scroll to-be-erased lines
     eraseResizeLines ();
 
     QTimer::singleShot (0, this, SLOT (slotContentsMoved ()));
 }
 
-// proteced slot
+// protected slot
 void kpViewScrollableContainer::slotContentsMoved ()
 {
+    m_contentsXSoon = m_contentsYSoon = -1;
+
     kpGrip *grip = docResizingGrip ();
 #if DEBUG_KP_VIEW_SCROLLABLE_CONTAINER
     kdDebug () << "kpViewScrollableContainer::slotContentsMoved()"
@@ -1304,6 +1327,19 @@ void kpViewScrollableContainer::mouseMoveEvent (QMouseEvent *e)
 }
 
 
+// protected virtual [base QScrollView]
+void kpViewScrollableContainer::contentsWheelEvent (QWheelEvent *e)
+{
+    e->ignore ();
+    
+    if (m_view)
+        m_view->wheelEvent (e);
+        
+    if (!e->isAccepted ())
+        QScrollView::contentsWheelEvent (e);
+}
+
+
 QRect kpViewScrollableContainer::noDragScrollRect () const
 {
     return QRect (DragScrollLeftTopMargin, DragScrollLeftTopMargin,
@@ -1341,5 +1377,13 @@ void kpViewScrollableContainer::paintEvent (QPaintEvent *e)
     QScrollView::paintEvent (e);
 }
 
+// protected virtual [base QScrollView]
+void kpViewScrollableContainer::resizeEvent (QResizeEvent *e)
+{
+    QScrollView::resizeEvent (e);
+
+    emit resized ();
+}
+
 
 #include <kpviewscrollablecontainer.moc>
