@@ -35,10 +35,12 @@ static inline double clip01(double x) {
 
 //------------------------------------------------------------------------
 
-static struct {
+struct gfxBlendModeName {
   char *name;
   GfxBlendMode mode;
-} gfxBlendModeNames[] = {
+};
+
+static gfxBlendModeName gfxBlendModeNames[] = {
   { "Normal",     gfxBlendNormal },
   { "Compatible", gfxBlendNormal },
   { "Multiply",   gfxBlendMultiply },
@@ -59,7 +61,7 @@ static struct {
 };
 
 #define nGfxBlendModeNames \
-          ((int)((sizeof(gfxBlendModeNames) / sizeof(char *))))
+          ((int)((sizeof(gfxBlendModeNames) / sizeof(gfxBlendModeName))))
 
 //------------------------------------------------------------------------
 
@@ -814,7 +816,8 @@ GfxColorSpace *GfxICCBasedColorSpace::parse(Array *arr) {
   cs = new GfxICCBasedColorSpace(nCompsA, altA, &iccProfileStreamA);
   if (dict->lookup("Range", &obj2)->isArray() &&
       obj2.arrayGetLength() == 2 * nCompsA) {
-    for (i = 0; i < nCompsA; ++i) {
+    // i should never be > 3 as rangeMin/rangeMax is a 4 elements array
+    for (i = 0; i < nCompsA && i < 4; ++i) {
       obj2.arrayGet(2*i, &obj3);
       cs->rangeMin[i] = obj3.getNum();
       obj3.free();
@@ -3208,6 +3211,9 @@ GfxImageColorMap::GfxImageColorMap(int bitsA, Object *decode,
   obj.free();
  err1:
   ok = gFalse;
+  for (k = 0; k < gfxColorMaxComps; ++k) {
+    lookup[k] = NULL;
+  }
 }
 
 GfxImageColorMap::GfxImageColorMap(GfxImageColorMap *colorMap) {
